""" Module for the null executable. Performs some very basic actions
    and utilizes pymygdala's LogHandler for logging. """

import os
import sys
import time
import random
import logging
import argparse

from ._version import ___version___ as version

_DESCRIPTION = """Workspaces null executable, a status capture test of the system. Version {}"""

logger = logging.getLogger("null")
logger.setLevel(logging.INFO)

class Null:
    def __init__(self, args, verbose):
        self.args = args
        if verbose:
            logger.setLevel(logging.DEBUG)
        self.args_to_funcs = {
            'print_error': self.print_error,
            'greeting': self.print_greeting,
            'exit-fail': self.exit_with_failure,
            'exit-random': self.exit_randomly,
            'nap': self.take_nap,
            'dump': self.dump_core
        }

    def print_error(self):
        print("in print_error")
        handler = logging.StreamHandler(stream=sys.stderr)
        logger.addHandler(handler)
        print(logger.getEffectiveLevel())
        logger.debug("STOP! YOU VIOLATED THE LAW! Pay the court a fine or serve your sentence.")
        logger.error("This is an error...")

    def print_greeting(self):
        logger.info("Hello, world!")
        logger.debug("And goodbye, world...")

    def exit_with_failure(self):
        logger.error("Error purposefully induced. Exiting with status code -1...")
        sys.exit(-1)

    def exit_randomly(self):
        status_code = random.randint(-50, 50)
        logger.debug("Exiting with status code {}".format(status_code))
        sys.exit(status_code)

    def take_nap(self):
        logger.debug("Going to sleep...")
        time.sleep(5)
        logger.debug("Waking up.")

    def dump_core(self):
        logger.debug("Aborting and dumping core...", stack_info=True)
        os.abort()

    def execute(self):
        """
        Executes command specified by CL arguments.
        """
        for arg, val in vars(self.args).items():
            if val and arg in self.args_to_funcs:
                self.args_to_funcs[arg]()

def make_arg_parser():
    """
    Creates an argparse arguments parser with appropriate options
    :return: Said argument parser
    """
    parser = argparse.ArgumentParser(description=_DESCRIPTION.format(version),
                                     formatter_class=argparse.RawTextHelpFormatter)
    options = parser.add_argument_group('options', 'settings for altering program behavior')
    options.add_argument('-v', '--verbose', action='store_true',
                         required=False, dest='verbose', default=False,
                         help='allow the program the gift of speech')

    functions = parser.add_mutually_exclusive_group(required=False)
    functions.add_argument('-pe', '--print-error', action='store_true',
                           required=False, dest='print-error', default=False,
                           help='print out aggressive message to stderr')
    functions.add_argument('-g', '--greeting', action='store_true',
                           required=False, dest='greeting', default=False,
                           help='print out a friendly greeting to stdout')
    functions.add_argument('-ef', '--exit-fail', action='store_true',
                           required=False, dest='exit-fail', default=False,
                           help='print error message and exit with status code -1')
    functions.add_argument('-er', '--exit-random', action='store_true',
                           required=False, dest='exit-random', default=False,
                           help='print error message and exit with random status code within [-50, 50]')
    functions.add_argument('-n', '--nap', action='store_true',
                           required=False, dest='nap', default=False,
                           help='take a short nap')
    functions.add_argument('-d', '--dump', action='store_true',
                           required=False, dest='dump', default=False,
                           help='abort program and dump core')
    return parser

def main():
    arg_parser = make_arg_parser()
    args = arg_parser.parse_args()
    handler = logging.StreamHandler(stream=sys.stdout)
    logger.addHandler(handler)

    executable = Null(args, args.verbose)
    executable.execute()

if __name__ == '__main__':
    main()